<#  
.SYNOPSIS  
	Sets the WindowsEmailAddress field to first.last@smtpdomain.com
	
.DESCRIPTION  
	This script is for NON EXCHANGE ENVIRONMENTS ONLY. It is designed to set the WindowsEmailAddress field of AD user accounts. It targets a specific OU and all child OUs.
	Note: It removes spaces in names to adhere to RFC 2821
	
.NOTES  
    Version      	   		: 1.2 - See changelog at http://www.ehloworld.com/605    
		Wish list						: better error trapping
												: verify running as admin before proceeding
												: demo mode
												: pipeline the smtp domain name
    Rights Required			: Rights to update user accounts
    Sched Task Req'd		: No
    Exchange Version		: N/A
    Lync Version				: N/A
    Author       				: Pat Richard, Lync MVP
    Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
    Dedicated Post			: http://www.ehloworld.com/240
    Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is provided AS IS 
    											without warranty of any kind. I disclaim all implied warranties including, without limitation, any implied 
    											warranties of merchantability or of fitness for a particular purpose. The entire risk arising out of the use 
    											or performance of the sample scripts and documentation remains with you. In no event shall I be liable for 
    											any damages whatsoever (including, without limitation, damages for loss of business profits, business 
    											interruption, loss of business information, or other pecuniary loss) arising out of the use of or inability 
    											to use the script or documentation. 
    Acknowlegements 		: (Get-Culture)	http://social.technet.microsoft.com/Forums/en/ITCG/thread/ab069b93-cf99-43a2-8226-014ceb0f05c5
    										: (OU picker) http://thepip3r.blogspot.com/2011/06/powershell-guis-active-directory.html
		Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
    Limitations					: 
    Known issues				: 
    										
.LINK  
	http://www.ehloworld.com/240
	
.EXAMPLE
	.\Set-WindowsEmailAddress.ps1 -SMTPDomain "contoso.com"
	
.INPUTS
	None. You cannot pipe objects to this script.
#>
#Requires -Version 2.0

[CmdletBinding(SupportsShouldProcess = $True)]
param(
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $false, Mandatory = $false)] 
	[string] $FilenameTranscript = $MyInvocation.MyCommand.Name + " " + (hostname)+ " {0:yyyy-MM-dd hh-mmtt}.log" -f (Get-Date),
	[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true)] 
	[ValidateNotNullOrEmpty()]
	[string] $SMTPDomainName
)
if (!($psise)){
	Start-Transcript -path .\$FilenameTranscript | Out-Null
}

function Set-ModuleStatus { 
	[CmdletBinding(SupportsShouldProcess = $True, SupportsPaging = $True)]
	param	(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage = "No module name specified!")] 
		[ValidateNotNullOrEmpty()]
		[string]$name
	)
	if(!(Get-Module -name "$name")) { 
		if(Get-Module -ListAvailable | Where-Object Name -eq "$name") { 
			Import-Module -Name "$name" 
			# module was imported
			return $true
		} else {
			# module was not available
			return $false
		}
	}else {
		# module was already imported
		# Write-Log "$name module already imported"
		return $true
	}
} # end function Set-ModuleStatus

function Remove-ScriptVariables($path) {  
	$result = Get-Content $path |  
	ForEach { if ( $_ -match '(\$.*?)\s*=') {      
			$matches[1]  | ? { $_ -notlike '*.*' -and $_ -notmatch 'result' -and $_ -notmatch 'env:'}  
		}  
	}  
	ForEach ($v in ($result | Sort-Object | Get-Unique)){		
		# Write-Host "Removing" $v.replace("$","")
		Remove-Variable ($v.replace("$","")) -ErrorAction SilentlyContinue
	}
} # end function Get-ScriptVariables

function Add-Node {
	param (
		$selectedNode,
		$name
	)
	$newNode = new-object System.Windows.Forms.TreeNode
 	$newNode.Name = $name
	$newNode.Text = $name
	$selectedNode.Nodes.Add($newNode) | Out-Null
	return $newNode  
} # end function Add-Node

function Get-NextLevel {
	param (
		$selectedNode,
		$dn
	)

	$OUs = Get-ADObject -Filter 'ObjectClass -eq "organizationalUnit" -or ObjectClass -eq "container"' -SearchScope OneLevel -SearchBase $dn
	
	If ($OUs -eq $null) {
		$node = Add-Node $selectedNode $dn
 	} Else {
		$node = Add-Node $selectedNode $dn
		# $OUs | % {
		$OUs | ForEach-Object {
   		Get-NextLevel $node $_.distinguishedName
		}
 	} 
} # end function Get-NextLevel

function Build-TreeView {
	if ($treeNodes)  {
  	$treeview1.Nodes.remove($treeNodes)
		$form1.Refresh()
  }

	$treeNodes = New-Object System.Windows.Forms.TreeNode
	$treeNodes.text = "Active Directory Hierarchy"
	$treeNodes.Name = "Active Directory Hierarchy"
	$treeNodes.Tag = "root"
	$treeView1.Nodes.Add($treeNodes) | Out-Null

	$treeView1.add_AfterSelect({
	$textbox1.Text = $this.SelectedNode.Name
	})
	
	#Generate Module nodes
  $OUs = Get-NextLevel $treeNodes $strDomainDN

	$treeNodes.Expand()  
} # end function Build-TreeView

#Generated Form Function  
function GenerateForm {
	#region Import the Assemblies
  # Import-Module ActiveDirectory
	[reflection.assembly]::loadwithpartialname("System.Windows.Forms") | Out-Null
  [reflection.assembly]::loadwithpartialname("System.Drawing") | Out-Null
  $objIPProperties = [System.Net.NetworkInformation.IPGlobalProperties]::GetIPGlobalProperties()
	$strDNSDomain = $objIPProperties.DomainName.toLower()
	$strDomainDN = $strDNSDomain.toString().split('.'); foreach ($strVal in $strDomainDN) {$strTemp += "dc=$strVal,"}; $strDomainDN = $strTemp.TrimEnd(",").toLower()
	#endregion
  
  #region Generated Form Objects
  $form1 = New-Object System.Windows.Forms.Form
  $treeView1 = New-Object System.Windows.Forms.TreeView
  $label1 = New-Object System.Windows.Forms.Label
 	$textbox1 = New-Object System.Windows.Forms.TextBox
 	$InitialFormWindowState = New-Object System.Windows.Forms.FormWindowState
  #endregion Generated Form Objects
  
  #----------------------------------------------
  #Generated Event Script Blocks
  #----------------------------------------------
  $button1_OnClick=
  {
  $form1.Close()
  
  }
  
  $OnLoadForm_StateCorrection=
  {Build-TreeView
  }
	
	#----------------------------------------------
  #region Generated Form Code
  $form1.Text = "Select source OU"
 	$form1.Name = "form1"
 	$form1.DataBindings.DefaultDataSourceUpdateMode = 0
  $form1.ClientSize = New-Object System.Drawing.Size(400,500)
  
  $treeView1.Size = New-Object System.Drawing.Size(350,375)
 	$treeView1.Name = "treeView1"
 	$treeView1.Location = New-Object System.Drawing.Size(15,15)
 	$treeView1.DataBindings.DefaultDataSourceUpdateMode = 0
  $treeView1.TabIndex = 0
  $form1.Controls.Add($treeView1)
 
 	$label1.Name = "label1"
 	$label1.Location = New-Object System.Drawing.Size(15,400)
 	$label1.Size = New-Object System.Drawing.Size(350,20)
 	$label1.Text = "Selected OU:"
	$form1.Controls.Add($label1)
  
  $textbox1.Name = "textbox1"
 	$textbox1.Location = New-Object System.Drawing.Size(15,425)
 	$textbox1.Size = New-Object System.Drawing.Size(350,20)
 	$textbox1.Text = ""
	$form1.Controls.Add($textbox1)

  $OKButton = New-Object System.Windows.Forms.Button
	$OKButton.Location = New-Object System.Drawing.Size(290,460)
	$OKButton.Size = New-Object System.Drawing.Size(75,23)
	$OKButton.Text = "OK"
	$OKButton.Add_Click({$global:ou=$textbox1.text;$form1.Close()})
	$form1.Controls.Add($OKButton)
	
	#endregion Generated Form Code
  
  #Save the initial state of the form
  $InitialFormWindowState = $form1.WindowState
  #Init the OnLoad event to correct the initial state of the form
  $form1.add_Load($OnLoadForm_StateCorrection)
  #Show the Form
  $form1.ShowDialog()| Out-Null

} # end function GenerateForm

if ((Set-ModuleStatus ActiveDirectory) -eq $false){
	$error.clear()
	Write-Host "Installing the Active Directory module..." -ForegroundColor yellow
	Set-ModuleStatus ServerManager
	Add-WindowsFeature RSAT-AD-PowerShell
	if ($error){
		Write-Host "Active Directory module could not be installed. Exiting..." -ForegroundColor red; 
		if ((!($psise)) -and $transcript){Stop-Transcript | Out-Null}
		exit
	}
}

GenerateForm
if ($ou){
	[object] $objUsers = Get-AdUser -filter * -ResultSetSize $null | ? {$_.DistinguishedName -match $ou -and $_.GivenName -ne $null -and $_.Surname -ne $null}
	if ($objUsers -ne $null){
		foreach ($objUser in $objUsers){
			[string] $strEmail = ((Get-Culture).TextInfo.ToTitleCase($objUser.GivenName.ToLower()).replace(" ","")) + "." + ((Get-Culture).TextInfo.ToTitleCase($objUser.Surname.ToLower()).replace(" ","")) + "@" + $SMTPDomainName
			# [string] $strEmail = ($objUser.GivenName.ToLower().replace(" ","")) + "." + ($objUser.Surname.ToLower().replace(" ","")) + "@" + $SMTPDomainName
			[string] $strSam = $objuser.samaccountname
			write-host $objUser.samaccountname -nonewline
			write-host " ($strEmail)" -foregroundcolor green
			Set-AdUser $strSam -emailaddress $strEmail
		}
	}else{
		Write-Host "No users found" -ForegroundColor yellow
	}

	Write-Host "`nDone" -foregroundcolor green
}

if (!($psise)){
	Stop-Transcript | Out-Null
}

Remove-ScriptVariables($MyInvocation.MyCommand.Name)
# SIG # Begin signature block
# MIIQBQYJKoZIhvcNAQcCoIIP9jCCD/ICAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUWnGZQFJMwncWlvMOLJZX1nCa
# 1TOggg1KMIIGnzCCBYegAwIBAgIQCSu3nvOdPAJrDHL+XciEEzANBgkqhkiG9w0B
# AQUFADBvMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMS4wLAYDVQQDEyVEaWdpQ2VydCBBc3N1cmVk
# IElEIENvZGUgU2lnbmluZyBDQS0xMB4XDTEzMDgyODAwMDAwMFoXDTE0MDkwNDEy
# MDAwMFowazELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAk1JMRkwFwYDVQQHExBTdGVy
# bGluZyBIZWlnaHRzMRkwFwYDVQQKExBJbm5lcnZhdGlvbiwgTExDMRkwFwYDVQQD
# ExBJbm5lcnZhdGlvbiwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
# AQEA8dXC+TSUZsMisaialKJLat7AnmOF0HE+U49duaBJp/N4IXhjUZM9sXRkR/X0
# KAbNauNpNrs3TLKNpO7/RUinzrzsg8IhtBVXSrK7c0HDqJ7vG181it2qQO1yB9uD
# UV8frjprnNcR1w0qW9wFqclKlkyOxgiCANVoT6qXO9IxPZT28h+tMfBCsU7sZALs
# 3kPRMCQ1ZywngVSqEQ9rBYV3jW4EgvPEOmuhtnwgVB1Alr2V1dVAtbNxw1A7fQ8w
# J6EQSNAPVJQq7304QKKgGaTXtA2TwAjO5+m9ZltD/s0wXWYs0V1r8pOJOo1r17l4
# z91OWLPP+yfxvlnm5F1IXZxCbQIDAQABo4IDOTCCAzUwHwYDVR0jBBgwFoAUe2jO
# KarAF75JeuHlP9an90WPNTIwHQYDVR0OBBYEFB9N9j4QNDPcSscWMAVDKCOcXS/G
# MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBzBgNVHR8EbDBq
# MDOgMaAvhi1odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vYXNzdXJlZC1jcy0yMDEx
# YS5jcmwwM6AxoC+GLWh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9hc3N1cmVkLWNz
# LTIwMTFhLmNybDCCAcQGA1UdIASCAbswggG3MIIBswYJYIZIAYb9bAMBMIIBpDA6
# BggrBgEFBQcCARYuaHR0cDovL3d3dy5kaWdpY2VydC5jb20vc3NsLWNwcy1yZXBv
# c2l0b3J5Lmh0bTCCAWQGCCsGAQUFBwICMIIBVh6CAVIAQQBuAHkAIAB1AHMAZQAg
# AG8AZgAgAHQAaABpAHMAIABDAGUAcgB0AGkAZgBpAGMAYQB0AGUAIABjAG8AbgBz
# AHQAaQB0AHUAdABlAHMAIABhAGMAYwBlAHAAdABhAG4AYwBlACAAbwBmACAAdABo
# AGUAIABEAGkAZwBpAEMAZQByAHQAIABDAFAALwBDAFAAUwAgAGEAbgBkACAAdABo
# AGUAIABSAGUAbAB5AGkAbgBnACAAUABhAHIAdAB5ACAAQQBnAHIAZQBlAG0AZQBu
# AHQAIAB3AGgAaQBjAGgAIABsAGkAbQBpAHQAIABsAGkAYQBiAGkAbABpAHQAeQAg
# AGEAbgBkACAAYQByAGUAIABpAG4AYwBvAHIAcABvAHIAYQB0AGUAZAAgAGgAZQBy
# AGUAaQBuACAAYgB5ACAAcgBlAGYAZQByAGUAbgBjAGUALjCBggYIKwYBBQUHAQEE
# djB0MCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wTAYIKwYB
# BQUHMAKGQGh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3Vy
# ZWRJRENvZGVTaWduaW5nQ0EtMS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0B
# AQUFAAOCAQEABTcbxhPxSJP7AKrF23wXbgelEGzjpewmm0BnHO9/V+9K4UE/3i5Z
# e3bX7OvUb/55zCHT4pvZl8/i8tuvFnrWZZcjGljT3W9SIVOuKNdlItXC8dvP67Hz
# OMZvSbdI5MU6sjd3vyYKznTMHG1Mz/x+BMh6rEnGsIaY4r8r4fkNfJTyTkww8AWh
# I16Pno3dWgKhfploPODtMxczJKA15VDFhQqM3oHHMojasXo5uGb5jlGiDFkZ3mDa
# /+gLUsJUjAzF27ZIy/jRgp0eMkqlgdxGa+gZUK73Ue6aTP2LW9SL/1fLYl8QkwRs
# MWG1faGdureT3Oh0EHH6UhREB/SxLJ+xLTCCBqMwggWLoAMCAQICEA+oSQYV1wCg
# viF2/cXsbb0wDQYJKoZIhvcNAQEFBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoT
# DERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UE
# AxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTExMDIxMTEyMDAwMFoX
# DTI2MDIxMDEyMDAwMFowbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNl
# cnQgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMTCCASIwDQYJKoZIhvcNAQEB
# BQADggEPADCCAQoCggEBAJx8+aCPCsqJS1OaPOwZIn8My/dIRNA/Im6aT/rO38bT
# JJH/qFKT53L48UaGlMWrF/R4f8t6vpAmHHxTL+WD57tqBSjMoBcRSxgg87e98tzL
# uIZARR9P+TmY0zvrb2mkXAEusWbpprjcBt6ujWL+RCeCqQPD/uYmC5NJceU4bU7+
# gFxnd7XVb2ZklGu7iElo2NH0fiHB5sUeyeCWuAmV+UuerswxvWpaQqfEBUd9YCvZ
# oV29+1aT7xv8cvnfPjL93SosMkbaXmO80LjLTBA1/FBfrENEfP6ERFC0jCo9dAz0
# eotyS+BWtRO2Y+k/Tkkj5wYW8CWrAfgoQebH1GQ7XasCAwEAAaOCA0MwggM/MA4G
# A1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDAzCCAcMGA1UdIASCAbow
# ggG2MIIBsgYIYIZIAYb9bAMwggGkMDoGCCsGAQUFBwIBFi5odHRwOi8vd3d3LmRp
# Z2ljZXJ0LmNvbS9zc2wtY3BzLXJlcG9zaXRvcnkuaHRtMIIBZAYIKwYBBQUHAgIw
# ggFWHoIBUgBBAG4AeQAgAHUAcwBlACAAbwBmACAAdABoAGkAcwAgAEMAZQByAHQA
# aQBmAGkAYwBhAHQAZQAgAGMAbwBuAHMAdABpAHQAdQB0AGUAcwAgAGEAYwBjAGUA
# cAB0AGEAbgBjAGUAIABvAGYAIAB0AGgAZQAgAEQAaQBnAGkAQwBlAHIAdAAgAEMA
# UAAvAEMAUABTACAAYQBuAGQAIAB0AGgAZQAgAFIAZQBsAHkAaQBuAGcAIABQAGEA
# cgB0AHkAIABBAGcAcgBlAGUAbQBlAG4AdAAgAHcAaABpAGMAaAAgAGwAaQBtAGkA
# dAAgAGwAaQBhAGIAaQBsAGkAdAB5ACAAYQBuAGQAIABhAHIAZQAgAGkAbgBjAG8A
# cgBwAG8AcgBhAHQAZQBkACAAaABlAHIAZQBpAG4AIABiAHkAIAByAGUAZgBlAHIA
# ZQBuAGMAZQAuMBIGA1UdEwEB/wQIMAYBAf8CAQAweQYIKwYBBQUHAQEEbTBrMCQG
# CCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQwYIKwYBBQUHMAKG
# N2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJv
# b3RDQS5jcnQwgYEGA1UdHwR6MHgwOqA4oDaGNGh0dHA6Ly9jcmwzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwOqA4oDaGNGh0dHA6Ly9j
# cmw0LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwHQYD
# VR0OBBYEFHtozimqwBe+SXrh5T/Wp/dFjzUyMB8GA1UdIwQYMBaAFEXroq/0ksuC
# MS1Ri6enIZ3zbcgPMA0GCSqGSIb3DQEBBQUAA4IBAQB7ch1k/4jIOsG36eepxIe7
# 25SS15BZM/orh96oW4AlPxOPm4MbfEPE5ozfOT7DFeyw2jshJXskwXJduEeRgRNG
# +pw/alE43rQly/Cr38UoAVR5EEYk0TgPJqFhkE26vSjmP/HEqpv22jVTT8nyPdNs
# 3CPtqqBNZwnzOoA9PPs2TJDndqTd8jq/VjUvokxl6ODU2tHHyJFqLSNPNzsZlBjU
# 1ZwQPNWxHBn/j8hrm574rpyZlnjRzZxRFVtCJnJajQpKI5JA6IbeIsKTOtSbaKbf
# KX8GuTwOvZ/EhpyCR0JxMoYJmXIJeUudcWn1Qf9/OXdk8YSNvosesn1oo6WQsQz/
# MYICJTCCAiECAQEwgYMwbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNl
# cnQgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMQIQCSu3nvOdPAJrDHL+XciE
# EzAJBgUrDgMCGgUAoHgwGAYKKwYBBAGCNwIBDDEKMAigAoAAoQKAADAZBgkqhkiG
# 9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIB
# FTAjBgkqhkiG9w0BCQQxFgQUXbBBwgwKpWikDlRMoE0iCreNhA4wDQYJKoZIhvcN
# AQEBBQAEggEA0VdFZ7524/C5q8gkyYNbi1pcB96/jN0QLTFYA9nLLMqU7Gz50ggI
# CqxG5D+6jgqeXmr1f2IHiP8uIxZotGQxw3hgdLxPSIYO9cITpwR9ZZl/8s9sxRNf
# QtmX7cV2mln+qPOvE7/KsHd+1n37pyiX4/Pvr67t4HqODxCWGqZ8rNCv90rsC3ev
# FlsKRXiE+P2rU7udSVJJO3HIXHLZ3In4BiaV2MTiOvt2Otxf1j2+M8+wkrnRxWQb
# Q1XDcVn4XUcRCDYigdai23YnnS84KEGAy44llXRxlmhQg+ijxqTlBPZpm2zQS4wG
# QQYqGuqo9+PLB5t7Q3KK6f5aa/UWZ9k7pg==
# SIG # End signature block
